<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\Client;
use App\Models\InvoiceItem;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class InvoiceController extends Controller
{
    public function index()
    {
        $invoices = Invoice::with('client')->latest()->paginate(10);
        return view('invoices.index', compact('invoices'));
    }

    public function create()
    {
        $clients = Client::all();
        $invoice_number = 'INV-' . date('Ymd') . '-' . rand(1000, 9999);
        return view('invoices.create', compact('clients', 'invoice_number'));
    }

    public function store(Request $request)
    {
        $validated = $request->validate([
            'invoice_number' => 'required|string|max:255|unique:invoices',
            'client_id' => 'required|exists:clients,id',
            'invoice_date' => 'required|date',
            'payment_type' => 'required|in:cash,transfer,credit',
            'gold_price' => 'required|numeric|min:0',
            'authorized_signature' => 'nullable|string|max:255',
            'items' => 'required|array|min:1',
            'items.*.details' => 'required|string',
            'items.*.weight' => 'required|numeric|min:0',
            'items.*.caliber' => 'required|string',
            'items.*.fees' => 'required|numeric|min:0',
        ]);

        try {
            DB::beginTransaction();

            $invoice = Invoice::create([
                'invoice_number' => $validated['invoice_number'],
                'client_id' => $validated['client_id'],
                'invoice_date' => $validated['invoice_date'],
                'payment_type' => $validated['payment_type'],
                'gold_price' => $validated['gold_price'],
                'authorized_signature' => $validated['authorized_signature'] ?? 'علي نزار محمد',
                'status' => 'new',
            ]);

            foreach ($validated['items'] as $itemData) {
                $item = new InvoiceItem([
                    'details' => $itemData['details'],
                    'weight' => $itemData['weight'],
                    'caliber' => $itemData['caliber'],
                    'fees' => $itemData['fees'],
                ]);

                $item->calculateTotal($validated['gold_price']);
                $invoice->items()->save($item);
            }

            $invoice->calculateTotals()->save();

            DB::commit();

            return redirect()->route('invoices.show', $invoice)
                ->with('success', 'Invoice created successfully.');
        } catch (\Exception $e) {
            DB::rollBack();
            return redirect()->back()->with('error', 'Error creating invoice: ' . $e->getMessage())->withInput();
        }
    }

    public function show(Invoice $invoice)
    {
        $invoice->load('client', 'items');
        return view('invoices.show', compact('invoice'));
    }

    public function print(Invoice $invoice)
    {
        $invoice->load('client', 'items');
        return view('invoices.print', compact('invoice'));
    }

    public function edit(Invoice $invoice)
    {
        $invoice->load('client', 'items');
        $clients = Client::all();
        return view('invoices.edit', compact('invoice', 'clients'));
    }

    public function update(Request $request, Invoice $invoice)
    {
        $validated = $request->validate([
            'invoice_number' => 'required|string|max:255|unique:invoices,invoice_number,' . $invoice->id,
            'client_id' => 'required|exists:clients,id',
            'invoice_date' => 'required|date',
            'payment_type' => 'required|in:cash,transfer,credit',
            'gold_price' => 'required|numeric|min:0',
            'authorized_signature' => 'nullable|string|max:255',
            'client_signature' => 'nullable|string|max:255',
            'items' => 'required|array|min:1',
            'items.*.id' => 'nullable|exists:invoice_items,id',
            'items.*.details' => 'required|string',
            'items.*.weight' => 'required|numeric|min:0',
            'items.*.caliber' => 'required|string',
            'items.*.fees' => 'required|numeric|min:0',
        ]);

        try {
            DB::beginTransaction();

            $invoice->update([
                'invoice_number' => $validated['invoice_number'],
                'client_id' => $validated['client_id'],
                'invoice_date' => $validated['invoice_date'],
                'payment_type' => $validated['payment_type'],
                'gold_price' => $validated['gold_price'],
                'authorized_signature' => $validated['authorized_signature'] ?? $invoice->authorized_signature,
                'client_signature' => $validated['client_signature'] ?? $invoice->client_signature,
            ]);

            // Delete removed items
            $keepItemIds = collect($validated['items'])->pluck('id')->filter()->toArray();
            $invoice->items()->whereNotIn('id', $keepItemIds)->delete();

            // Update or create items
            foreach ($validated['items'] as $itemData) {
                if (isset($itemData['id'])) {
                    $item = InvoiceItem::find($itemData['id']);
                    $item->update([
                        'details' => $itemData['details'],
                        'weight' => $itemData['weight'],
                        'caliber' => $itemData['caliber'],
                        'fees' => $itemData['fees'],
                    ]);
                    $item->calculateTotal($validated['gold_price'])->save();
                } else {
                    $item = new InvoiceItem([
                        'details' => $itemData['details'],
                        'weight' => $itemData['weight'],
                        'caliber' => $itemData['caliber'],
                        'fees' => $itemData['fees'],
                    ]);
                    $item->calculateTotal($validated['gold_price']);
                    $invoice->items()->save($item);
                }
            }

            $invoice->calculateTotals()->save();

            DB::commit();

            return redirect()->route('invoices.show', $invoice)
                ->with('success', 'Invoice updated successfully.');
        } catch (\Exception $e) {
            DB::rollBack();
            return redirect()->back()->with('error', 'Error updating invoice: ' . $e->getMessage())->withInput();
        }
    }

    public function destroy(Invoice $invoice)
    {
        $invoice->delete();
        return redirect()->route('invoices.index')
            ->with('success', 'Invoice deleted successfully.');
    }
}